/*
 * 		Ar-Du Arduino023 code
 * 		By: Fizikus
 *  		19/06/2012
 */


#include <JPEGCamera.h>
#include <NewSoftSerial.h>
#include <Servo.h>

//Create an instance of the camera
JPEGCamera camera;

Servo szervo;
unsigned long EchoTime = 0;  // Echo time in ns  
unsigned long Tavolsag = 500;  // distance in mm
//Pin0 (Rx) : BT module Tx Pin
//Pin1 (Tx) : BT module Rx Pin
//Pin2 : NewSoftSerial Rx - JPEG camera Tx
//Pin3 : NewSoftSerial Tx - JPEG camera Rx 
#define M1 4 //Pin4 : Motor1 diredtion
#define EN1 5 //Pin5 : Motor1 speed (PWM)
#define EN2 6  //Pin6 : Motor2 speed (PWM)  
#define M2 7 //Pin7 : Motor2 diredtion
#define SoundModule 8 //Pin8 : Sound Module
//Pin9 : Head rotating servo
#define BLED 10 //Pin10 : Blue LED                        
#define RLED 11  //Pin11 : Red LED
#define TriggerPin 12 //Pin12 :Ultrasound module - Trigger pin
#define EchoPin 13   //Pin13 :Ultrasound module - Echo pin
int pwm = 0;
boolean EndFlag = 0;
                               
//Create a character array to store the cameras response
char response[32];
//Count store the number of characters in the response string.
unsigned int count=0;
//Size will be set to the size of the jpeg image.
int size=0;
//keep track of the data address being read from the camera
int address=0;
//eof is a flag for the sketch to determine when the end of a // file is detected while reading the data from the camera.
int eof=0;
int takeNewPhoto = 0;
int inByte = 0;         //incoming serial byte
boolean takingPhoto = 0;  // now taking a photo or not 
boolean sendingPhoto = 0;  // now sending a photo or not
boolean motorElore = 0;   // now the motors go forward or not


void setup()
{ 
  pinMode(TriggerPin, OUTPUT); 
  pinMode(EchoPin, INPUT);   
  pinMode(RLED, OUTPUT);   
  pinMode(BLED, OUTPUT);
  pinMode(SoundModule, OUTPUT);
  pinMode(EN1, OUTPUT);   
  pinMode(EN2, OUTPUT);
  pinMode(M1, OUTPUT);   
  pinMode(M2, OUTPUT); 
  Motor1(0); // motor1 stop
  Motor2(0); // motor2 stop
  digitalWrite(SoundModule,HIGH);
  digitalWrite(RLED,HIGH);
  digitalWrite(BLED,LOW);
  szervo.attach(9);
  szervo.write(90); 
  Serial.begin(115200);
  camera.begin();
  count=camera.reset(response);
  delay(3000); 
}

void loop() 
{ 
 Tavolsag = UHtavmeres();
 //if something is in front of ArDu, and closer than 15cm:STOP
      if (Tavolsag < 150 && motorElore == 1) 
       { 
         Motor1(0); // motor1 stop
         Motor2(0); // motor2 stop
       } 
 if (Serial.available() > 0) 
  {
    // get incoming byte:
    int inByte = Serial.read();
    if(inByte == '1')
    {
      pwm = 60;   // motor speed = 60%
    } 
    else if(inByte == '2')
    {
     pwm = 80;   // motor speed = 80%
    }
    else if(inByte == '3')
    {
     pwm = 100;   // motor speed = 100%
    }
    else if(inByte == 'b')
    {
     motorElore = 0;
     Motor1(-pwm);  
     Motor2(pwm);   // turn left
    }
    else if(inByte == 'j')
    {
     motorElore = 0;
     Motor1(pwm);
     Motor2(-pwm); // turn right
    }
    else if(inByte == 'e')
    {
      motorElore = 1;
      Tavolsag = UHtavmeres();
      if (Tavolsag > 150)   // if distance greater than 15cm
       { 
         Motor1(pwm);
         Motor2(pwm); // forward
       }
    }
    else if(inByte == 'h')
    {
     Motor1(-pwm);
     Motor2(-pwm); // backward
    }
    else if(inByte == 's')
    {
     Motor1(0);
     Motor2(0); // stop
    }
    else if(inByte == 'a')
    {
     szervo.write(180); // look left
    }
    else if(inByte == 'd')
    {
     szervo.write(0); // look right
    }
    else if(inByte == 'w')
    {
     szervo.write(90); // look forward
    }
    else if(inByte == 'v')
    {
     Sound();   // sound and lights
    }
    else if(inByte == 'k')
    {
      takeAndSendPhoto();  // take and send photo
    }
  }
}

void takeAndSendPhoto()
{
  if(takingPhoto)return;
  address = 0;
  eof = 0;
  size = 0;
  count = 0;
  takingPhoto = true;
  //Take a picture
  count=camera.takePicture(response);
  //Get the size of the picture
  count = camera.getSize(response, &size);
  //keep reading data until we've read 'size' data.
  while(address < size)
  {
    //Read the data starting at the current address.
    count=camera.readData(response, address);
    //Store all of the data that we read
    for(int i=0; i<count; i++)
    {
      if((response[i] == (char)0xD9) && (response[i-1]==(char)0xFF)) eof=1;
      //Save the data to the PC
      Serial.print( (char) response[i] , BYTE);
      //If end of file, stop reading data
      if(eof==1) break;
    }
    address+=count;
    if(eof==1)
    {
      Serial.print("EOF");
      break;
    }
  } 
  camera.stopPictures(response);
  takingPhoto = false;
}

int UHtavmeres()  // Ultrasound distance measurement
{
  digitalWrite(TriggerPin, HIGH);  // 10 us Trigger signal 
  delayMicroseconds(10);
  digitalWrite(TriggerPin, LOW);
  // measuring the Echo time 
  EchoTime = pulseIn(EchoPin, HIGH);    
  //distance in mm = 10 * EchoTime / 58
  return 10*EchoTime/58;
}

void Motor1(int sebesseg)
{   
    if (sebesseg>0)   // if speed is greater than 0 (0 - 100%)
	{
	digitalWrite(M1,HIGH);  // motor1 forward
    	analogWrite(EN1,sebesseg*255/100);  // motor1 speed
	} 
	else  // if speed is less than 0 (-100% - 0%)
	{
	digitalWrite(M1,LOW);  // motor1 backward
    	analogWrite(EN1,abs(sebesseg)*255/100); // motor1 speed
	}
} 

void Motor2(int sebesseg) 
{   
    if (sebesseg>0)
	{
	digitalWrite(M2,HIGH);
    	analogWrite(EN2,sebesseg*255/100);
	} 
	else
	{
	digitalWrite(M2,LOW);
    	analogWrite(EN2,abs(sebesseg)*255/100);
	}
}

void Sound()   // signal to sound module and switching LEDs
{
      digitalWrite(SoundModule,LOW);    
      delay(50);
      digitalWrite(SoundModule,HIGH);
      delay(200); 
      digitalWrite(RLED,LOW);
      digitalWrite(BLED,HIGH);
      delay(250);
      digitalWrite(RLED,HIGH);
      digitalWrite(BLED,LOW);
      delay(125);
      digitalWrite(RLED,LOW);
      digitalWrite(BLED,HIGH);
      delay(125);
      digitalWrite(RLED,HIGH);
      digitalWrite(BLED,LOW);
      delay(125);
      digitalWrite(RLED,LOW);
      digitalWrite(BLED,HIGH);
      delay(250);
      digitalWrite(RLED,HIGH);
      digitalWrite(BLED,LOW);
      delay(125);
      digitalWrite(RLED,LOW);
      digitalWrite(BLED,HIGH);
      delay(125);
      digitalWrite(RLED,HIGH);
      digitalWrite(BLED,LOW);
      delay(125);
      digitalWrite(RLED,LOW);
      digitalWrite(BLED,HIGH);
      delay(125);
      digitalWrite(RLED,HIGH);
      digitalWrite(BLED,LOW);
      delay(125);
      digitalWrite(RLED,LOW);
      digitalWrite(BLED,HIGH);
      delay(200);
      digitalWrite(RLED,HIGH);
      digitalWrite(BLED,LOW);
      delay(125);
      digitalWrite(RLED,LOW);
      digitalWrite(BLED,HIGH);
      delay(125);
      digitalWrite(RLED,HIGH);
      digitalWrite(BLED,LOW);
      delay(125);
      digitalWrite(RLED,LOW);
      digitalWrite(BLED,HIGH);
      delay(125);
      digitalWrite(RLED,HIGH);
      digitalWrite(BLED,LOW);
      delay(125);
      digitalWrite(RLED,LOW);
      digitalWrite(BLED,HIGH);
      delay(500);
      digitalWrite(RLED,HIGH);
      digitalWrite(BLED,LOW);
      delay(125); 
      digitalWrite(RLED,LOW);
      digitalWrite(BLED,HIGH);
      delay(125);
      digitalWrite(RLED,HIGH);
      digitalWrite(BLED,LOW);   
}

